@@ -27,13 +27,10 @@ static FUNCTION_COUNT: AtomicUsize = AtomicUsize::new(0);
 
 // TODO: impl Fn
 #[derive(Clone)]
-pub(crate) struct Builtin(
-    pub fn(CallArgs, &mut Parser<'_>) -> SassResult<Value>,
-    usize,
-);
+pub(crate) struct Builtin(pub fn(CallArgs, &mut Parser) -> SassResult<Value>, usize);
 
 impl Builtin {
-    pub fn new(body: fn(CallArgs, &mut Parser<'_>) -> SassResult<Value>) -> Builtin {
+    pub fn new(body: fn(CallArgs, &mut Parser) -> SassResult<Value>) -> Builtin {
         let count = FUNCTION_COUNT.fetch_add(1, Ordering::Relaxed);
         Self(body, count)
     }
