@@ -1,20 +1,18 @@
 use codemap::Spanned;
 use num_traits::cast::ToPrimitive;
-use peekmore::PeekMore;
 
 use crate::{
     common::Identifier,
     error::SassResult,
+    lexer::Lexer,
     parse::{ContextFlags, Parser, Stmt},
     unit::Unit,
-    utils::{
-        peek_ident_no_interpolation, read_until_closing_curly_brace, read_until_open_curly_brace,
-    },
+    utils::{read_until_closing_curly_brace, read_until_open_curly_brace},
     value::{Number, Value},
     Token,
 };
 
-impl<'a> Parser<'a> {
+impl<'a, 'b> Parser<'a, 'b> {
     pub(super) fn parse_if(&mut self) -> SassResult<Vec<Stmt>> {
         self.whitespace_or_comment();
 
@@ -29,11 +27,10 @@ impl<'a> Parser<'a> {
             return Err(("expected \"}\".", self.span_before).into());
         }
 
-        self.whitespace_or_comment();
-
         if init_cond.is_true() {
             self.scopes.enter_new_scope();
             found_true = true;
+
             body = Parser {
                 toks: self.toks,
                 map: self.map,
@@ -53,6 +50,7 @@ impl<'a> Parser<'a> {
                 module_config: self.module_config,
             }
             .parse_stmt()?;
+
             self.scopes.exit_scope();
         } else {
             self.throw_away_until_closing_curly_brace()?;
@@ -60,27 +58,27 @@ impl<'a> Parser<'a> {
 
         loop {
             self.whitespace_or_comment();
-            if let Some(Token { kind: '@', pos }) = self.toks.peek().copied() {
-                self.toks.peek_forward(1);
-                let ident = peek_ident_no_interpolation(self.toks, false, pos)?;
-                if ident.as_str() != "else" {
-                    self.toks.reset_cursor();
-                    break;
-                }
-                self.toks.truncate_iterator_to_cursor();
-            } else {
+
+            let start = self.toks.cursor();
+
+            if !self.consume_char_if_exists('@') || !self.scan_identifier("else", false) {
+                self.toks.set_cursor(start);
                 break;
             }
+
             self.whitespace_or_comment();
-            if let Some(tok) = self.toks.peek().copied() {
+            if let Some(tok) = self.toks.peek() {
                 match tok.kind {
-                    'i' if matches!(
-                        self.toks.peek_forward(1),
-                        Some(Token { kind: 'f', .. }) | Some(Token { kind: 'F', .. })
-                    ) =>
-                    {
-                        self.toks.next();
-                        self.toks.next();
+                    'i' | 'I' | '\\' => {
+                        self.span_before = tok.pos;
+                        let mut ident = self.parse_identifier_no_interpolation(false)?;
+
+                        ident.node.make_ascii_lowercase();
+
+                        if ident.node != "if" {
+                            return Err(("expected \"{\".", ident.span).into());
+                        }
+
                         let cond = if found_true {
                             self.throw_away_until_open_curly_brace()?;
                             false
@@ -89,6 +87,7 @@ impl<'a> Parser<'a> {
                             self.expect_char('{')?;
                             v
                         };
+
                         if cond {
                             found_true = true;
                             self.scopes.enter_new_scope();
@@ -111,6 +110,7 @@ impl<'a> Parser<'a> {
                                 module_config: self.module_config,
                             }
                             .parse_stmt()?;
+
                             self.scopes.exit_scope();
                         } else {
                             self.throw_away_until_closing_curly_brace()?;
@@ -178,26 +178,31 @@ impl<'a> Parser<'a> {
         }
         self.whitespace_or_comment();
 
-        let from_val = self.parse_value(false, &|toks| match toks.peek() {
-            Some(Token { kind: 't', pos })
-            | Some(Token { kind: 'T', pos })
-            | Some(Token { kind: '\\', pos }) => {
-                let span = *pos;
-                let mut ident = match peek_ident_no_interpolation(toks, false, span) {
+        let from_val = self.parse_value(false, &|parser| match parser.toks.peek() {
+            Some(Token { kind: 't', .. })
+            | Some(Token { kind: 'T', .. })
+            | Some(Token { kind: '\\', .. }) => {
+                let start = parser.toks.cursor();
+
+                let mut ident = match parser.parse_identifier_no_interpolation(false) {
                     Ok(s) => s,
                     Err(..) => return false,
                 };
+
                 ident.node.make_ascii_lowercase();
+
                 let v = matches!(ident.node.to_ascii_lowercase().as_str(), "to" | "through");
-                toks.reset_cursor();
+
+                parser.toks.set_cursor(start);
+
                 v
             }
             Some(..) | None => false,
         })?;
 
-        let through = if self.scan_identifier("through") {
+        let through = if self.scan_identifier("through", true) {
             1
-        } else if self.scan_identifier("to") {
+        } else if self.scan_identifier("to", true) {
             0
         } else {
             return Err(("Expected \"to\" or \"through\".", self.span_before).into());
@@ -206,7 +211,7 @@ impl<'a> Parser<'a> {
         let from = match from_val.node {
             Value::Dimension(Some(n), ..) => match n.to_i32() {
                 Some(std::i32::MAX) | Some(std::i32::MIN) | None => {
-                    return Err((format!("{} is not an int.", n), from_val.span).into())
+                    return Err((format!("{} is not an int.", n.inspect()), from_val.span).into())
                 }
                 Some(v) => v,
             },
@@ -224,14 +229,17 @@ impl<'a> Parser<'a> {
         let to = match to_val.node {
             Value::Dimension(Some(n), ..) => match n.to_i32() {
                 Some(std::i32::MAX) | Some(std::i32::MIN) | None => {
-                    return Err((format!("{} is not an int.", n), to_val.span).into())
+                    return Err((format!("{} is not an int.", n.inspect()), to_val.span).into())
                 }
                 Some(v) => v,
             },
             Value::Dimension(None, ..) => return Err(("NaN is not an int.", from_val.span).into()),
             v => {
                 return Err((
-                    format!("{} is not a number.", v.to_css_string(to_val.span)?),
+                    format!(
+                        "{} is not a number.",
+                        v.to_css_string(to_val.span, self.options.is_compressed())?
+                    ),
                     to_val.span,
                 )
                     .into())
@@ -266,7 +274,7 @@ impl<'a> Parser<'a> {
             );
             if self.flags.in_function() {
                 let these_stmts = Parser {
-                    toks: &mut body.clone().into_iter().peekmore(),
+                    toks: &mut Lexer::new_ref(&body),
                     map: self.map,
                     path: self.path,
                     scopes: self.scopes,
@@ -290,7 +298,7 @@ impl<'a> Parser<'a> {
             } else {
                 stmts.append(
                     &mut Parser {
-                        toks: &mut body.clone().into_iter().peekmore(),
+                        toks: &mut Lexer::new_ref(&body),
                         map: self.map,
                         path: self.path,
                         scopes: self.scopes,
@@ -338,12 +346,12 @@ impl<'a> Parser<'a> {
         });
 
         let mut stmts = Vec::new();
-        let mut val = self.parse_value_from_vec(cond.clone(), true)?;
+        let mut val = self.parse_value_from_vec(&cond, true)?;
         self.scopes.enter_new_scope();
         while val.node.is_true() {
             if self.flags.in_function() {
                 let these_stmts = Parser {
-                    toks: &mut body.clone().into_iter().peekmore(),
+                    toks: &mut Lexer::new_ref(&body),
                     map: self.map,
                     path: self.path,
                     scopes: self.scopes,
@@ -367,7 +375,7 @@ impl<'a> Parser<'a> {
             } else {
                 stmts.append(
                     &mut Parser {
-                        toks: &mut body.clone().into_iter().peekmore(),
+                        toks: &mut Lexer::new_ref(&body),
                         map: self.map,
                         path: self.path,
                         scopes: self.scopes,
@@ -387,7 +395,7 @@ impl<'a> Parser<'a> {
                     .parse_stmt()?,
                 );
             }
-            val = self.parse_value_from_vec(cond.clone(), true)?;
+            val = self.parse_value_from_vec(&cond, true)?;
         }
         self.scopes.exit_scope();
 
@@ -424,7 +432,7 @@ impl<'a> Parser<'a> {
         self.whitespace_or_comment();
         let iter_val_toks = read_until_open_curly_brace(self.toks)?;
         let iter = self
-            .parse_value_from_vec(iter_val_toks, true)?
+            .parse_value_from_vec(&iter_val_toks, true)?
             .node
             .as_list();
         self.toks.next();
@@ -455,7 +463,7 @@ impl<'a> Parser<'a> {
 
             if self.flags.in_function() {
                 let these_stmts = Parser {
-                    toks: &mut body.clone().into_iter().peekmore(),
+                    toks: &mut Lexer::new_ref(&body),
                     map: self.map,
                     path: self.path,
                     scopes: self.scopes,
@@ -479,7 +487,7 @@ impl<'a> Parser<'a> {
             } else {
                 stmts.append(
                     &mut Parser {
-                        toks: &mut body.clone().into_iter().peekmore(),
+                        toks: &mut Lexer::new_ref(&body),
                         map: self.map,
                         path: self.path,
                         scopes: self.scopes,
