@@ -1,4 +1,4 @@
-use std::{iter::Iterator, mem, vec::IntoIter};
+use std::{iter::Iterator, mem};
 
 use num_bigint::BigInt;
 use num_rational::{BigRational, Rational64};
@@ -6,15 +6,14 @@ use num_traits::{pow, One, ToPrimitive};
 
 use codemap::{Span, Spanned};
 
-use peekmore::{PeekMore, PeekMoreIterator};
-
 use crate::{
     builtin::GLOBAL_FUNCTIONS,
     color::{Color, NAMED_COLORS},
     common::{unvendor, Brackets, Identifier, ListSeparator, Op, QuoteKind},
     error::SassResult,
+    lexer::Lexer,
     unit::Unit,
-    utils::{eat_whole_number, is_name, IsWhitespace, ParsedNumber},
+    utils::{is_name, IsWhitespace, ParsedNumber},
     value::{Number, SassFunction, SassMap, Value},
     Token,
 };
@@ -46,14 +45,17 @@ impl IsWhitespace for IntermediateValue {
     }
 }
 
-impl<'a> Parser<'a> {
+/// We parse a value until the predicate returns true
+type Predicate<'a> = &'a dyn Fn(&mut Parser<'_, '_>) -> bool;
+
+impl<'a, 'b> Parser<'a, 'b> {
     /// Parse a value from a stream of tokens
     ///
     /// This function will cease parsing if the predicate returns true.
     pub(crate) fn parse_value(
         &mut self,
         in_paren: bool,
-        predicate: &dyn Fn(&mut PeekMoreIterator<IntoIter<Token>>) -> bool,
+        predicate: Predicate<'_>,
     ) -> SassResult<Spanned<Value>> {
         self.whitespace();
 
@@ -62,10 +64,10 @@ impl<'a> Parser<'a> {
             | Some(Token { kind: ';', .. })
             | Some(Token { kind: '{', .. })
             | None => return Err(("Expected expression.", self.span_before).into()),
-            Some(Token { pos, .. }) => *pos,
+            Some(Token { pos, .. }) => pos,
         };
 
-        if predicate(self.toks) {
+        if predicate(self) {
             return Err(("Expected expression.", span).into());
         }
 
@@ -169,11 +171,11 @@ impl<'a> Parser<'a> {
 
     pub(crate) fn parse_value_from_vec(
         &mut self,
-        toks: Vec<Token>,
+        toks: &[Token],
         in_paren: bool,
     ) -> SassResult<Spanned<Value>> {
         Parser {
-            toks: &mut toks.into_iter().peekmore(),
+            toks: &mut Lexer::new_ref(toks),
             map: self.map,
             path: self.path,
             scopes: self.scopes,
@@ -196,8 +198,7 @@ impl<'a> Parser<'a> {
     #[allow(clippy::eval_order_dependence)]
     fn parse_module_item(
         &mut self,
-        module: &str,
-        mut module_span: Span,
+        mut module: Spanned<Identifier>,
     ) -> SassResult<Spanned<IntermediateValue>> {
         Ok(
             IntermediateValue::Value(if self.consume_char_if_exists('$') {
@@ -205,9 +206,9 @@ impl<'a> Parser<'a> {
                     .parse_identifier_no_interpolation(false)?
                     .map_node(|i| i.into());
 
-                module_span = module_span.merge(var.span);
+                module.span = module.span.merge(var.span);
 
-                let value = self.modules.get(module.into(), module_span)?.get_var(var)?;
+                let value = self.modules.get(module.node, module.span)?.get_var(var)?;
                 HigherIntermediateValue::Literal(value.clone())
             } else {
                 let fn_name = self
@@ -216,7 +217,7 @@ impl<'a> Parser<'a> {
 
                 let function = self
                     .modules
-                    .get(module.into(), module_span)?
+                    .get(module.node, module.span)?
                     .get_fn(fn_name)?
                     .ok_or(("Undefined function.", fn_name.span))?;
 
@@ -224,9 +225,9 @@ impl<'a> Parser<'a> {
 
                 let call_args = self.parse_call_args()?;
 
-                HigherIntermediateValue::Function(function, call_args)
+                HigherIntermediateValue::Function(function, call_args, Some(module))
             })
-            .span(module_span),
+            .span(module.span),
         )
     }
 
@@ -236,16 +237,16 @@ impl<'a> Parser<'a> {
         lower: String,
     ) -> SassResult<Spanned<IntermediateValue>> {
         if lower == "min" || lower == "max" {
+            let start = self.toks.cursor();
             match self.try_parse_min_max(&lower, true)? {
                 Some(val) => {
-                    self.toks.truncate_iterator_to_cursor();
                     return Ok(IntermediateValue::Value(HigherIntermediateValue::Literal(
                         Value::String(val, QuoteKind::None),
                     ))
                     .span(self.span_before));
                 }
                 None => {
-                    self.toks.reset_cursor();
+                    self.toks.set_cursor(start);
                 }
             }
         }
@@ -258,6 +259,7 @@ impl<'a> Parser<'a> {
                     return Ok(IntermediateValue::Value(HigherIntermediateValue::Function(
                         SassFunction::Builtin(f.clone(), as_ident),
                         self.parse_call_args()?,
+                        None,
                     ))
                     .span(self.span_before));
                 }
@@ -270,9 +272,20 @@ impl<'a> Parser<'a> {
                     }
                     "url" => match self.try_parse_url()? {
                         Some(val) => s = val,
-                        None => s.push_str(&self.parse_call_args()?.to_css_string()?),
+                        None => s.push_str(
+                            &self
+                                .parse_call_args()?
+                                .to_css_string(self.options.is_compressed())?,
+                        ),
                     },
-                    _ => s.push_str(&self.parse_call_args()?.to_css_string()?),
+                    "clamp" if lower == "clamp" => {
+                        self.parse_calc_args(&mut s)?;
+                    }
+                    _ => s.push_str(
+                        &self
+                            .parse_call_args()?
+                            .to_css_string(self.options.is_compressed())?,
+                    ),
                 }
 
                 return Ok(IntermediateValue::Value(HigherIntermediateValue::Literal(
@@ -284,14 +297,14 @@ impl<'a> Parser<'a> {
 
         let call_args = self.parse_call_args()?;
         Ok(
-            IntermediateValue::Value(HigherIntermediateValue::Function(func, call_args))
+            IntermediateValue::Value(HigherIntermediateValue::Function(func, call_args, None))
                 .span(self.span_before),
         )
     }
 
     fn parse_ident_value(
         &mut self,
-        predicate: &dyn Fn(&mut PeekMoreIterator<IntoIter<Token>>) -> bool,
+        predicate: Predicate<'_>,
     ) -> SassResult<Spanned<IntermediateValue>> {
         let Spanned { node: mut s, span } = self.parse_identifier()?;
 
@@ -321,9 +334,12 @@ impl<'a> Parser<'a> {
                     return self.parse_fn_call(s, lower);
                 }
                 Some(Token { kind: '.', .. }) => {
-                    if !predicate(self.toks) {
+                    if !predicate(self) {
                         self.toks.next();
-                        return self.parse_module_item(&s, span);
+                        return self.parse_module_item(Spanned {
+                            node: s.into(),
+                            span,
+                        });
                     }
                 }
                 _ => {}
@@ -361,26 +377,38 @@ impl<'a> Parser<'a> {
         }
     }
 
-    fn parse_number(
-        &mut self,
-        predicate: &dyn Fn(&mut PeekMoreIterator<IntoIter<Token>>) -> bool,
-    ) -> SassResult<Spanned<ParsedNumber>> {
-        let mut span = self.toks.peek().unwrap().pos;
-        let mut whole = eat_whole_number(self.toks);
+    pub(crate) fn parse_whole_number(&mut self) -> String {
+        let mut buf = String::new();
 
-        if self.toks.peek().is_none() || predicate(self.toks) {
+        while let Some(c) = self.toks.peek() {
+            if !c.kind.is_ascii_digit() {
+                break;
+            }
+
+            let tok = self.toks.next().unwrap();
+            buf.push(tok.kind);
+        }
+
+        buf
+    }
+
+    fn parse_number(&mut self, predicate: Predicate<'_>) -> SassResult<Spanned<ParsedNumber>> {
+        let mut span = self.toks.peek().unwrap().pos;
+        let mut whole = self.parse_whole_number();
+
+        if self.toks.peek().is_none() || predicate(self) {
             return Ok(Spanned {
                 node: ParsedNumber::new(whole, 0, String::new(), true),
                 span,
             });
         }
 
-        let next_tok = *self.toks.peek().unwrap();
+        let next_tok = self.toks.peek().unwrap();
 
         let dec_len = if next_tok.kind == '.' {
             self.toks.next();
 
-            let dec = eat_whole_number(self.toks);
+            let dec = self.parse_whole_number();
             if dec.is_empty() {
                 return Err(("Expected digit.", next_tok.pos()).into());
             }
@@ -394,41 +422,48 @@ impl<'a> Parser<'a> {
 
         let mut times_ten = String::new();
         let mut times_ten_is_postive = true;
+
         if let Some(Token { kind: 'e', .. }) | Some(Token { kind: 'E', .. }) = self.toks.peek() {
-            if let Some(&tok) = self.toks.peek_next() {
-                if tok.kind == '-' {
-                    self.toks.next();
-                    times_ten_is_postive = false;
+            if let Some(tok) = self.toks.peek_next() {
+                match tok.kind {
+                    '-' => {
+                        self.toks.next();
+                        self.toks.next();
+                        times_ten_is_postive = false;
 
-                    self.toks.next();
-                    times_ten = eat_whole_number(self.toks);
+                        times_ten = self.parse_whole_number();
 
-                    if times_ten.is_empty() {
-                        return Err(
-                            ("Expected digit.", self.toks.peek().unwrap_or(&tok).pos).into()
-                        );
-                    } else if times_ten.len() > 2 {
-                        return Err((
-                            "Exponent too negative.",
-                            self.toks.peek().unwrap_or(&tok).pos,
-                        )
-                            .into());
+                        if times_ten.is_empty() {
+                            return Err(
+                                ("Expected digit.", self.toks.peek().unwrap_or(tok).pos).into()
+                            );
+                        } else if times_ten.len() > 2 {
+                            return Err((
+                                "Exponent too negative.",
+                                self.toks.peek().unwrap_or(tok).pos,
+                            )
+                                .into());
+                        }
                     }
-                } else if matches!(tok.kind, '0'..='9') {
-                    self.toks.next();
-                    times_ten = eat_whole_number(self.toks);
+                    '0'..='9' => {
+                        self.toks.next();
+                        times_ten = self.parse_whole_number();
 
-                    if times_ten.len() > 2 {
-                        return Err(
-                            ("Exponent too large.", self.toks.peek().unwrap_or(&tok).pos).into(),
-                        );
+                        if times_ten.len() > 2 {
+                            return Err((
+                                "Exponent too large.",
+                                self.toks.peek().unwrap_or(tok).pos,
+                            )
+                                .into());
+                        }
                     }
+                    _ => {}
                 }
             }
         }
 
-        if let Ok(Some(Token { pos, .. })) = self.toks.peek_previous() {
-            span = span.merge(*pos);
+        if let Some(Token { pos, .. }) = self.toks.peek_previous() {
+            span = span.merge(pos);
         }
 
         self.toks.reset_cursor();
@@ -445,7 +480,7 @@ impl<'a> Parser<'a> {
         self.whitespace_or_comment();
 
         Ok(if let Some(Token { kind: ']', pos }) = self.toks.peek() {
-            span = span.merge(*pos);
+            span = span.merge(pos);
             self.toks.next();
             IntermediateValue::Value(HigherIntermediateValue::Literal(Value::List(
                 Vec::new(),
@@ -455,8 +490,8 @@ impl<'a> Parser<'a> {
             .span(span)
         } else {
             // todo: we don't know if we're `in_paren` here
-            let inner = self.parse_value(false, &|toks| {
-                matches!(toks.peek(), Some(Token { kind: ']', .. }))
+            let inner = self.parse_value(false, &|parser| {
+                matches!(parser.toks.peek(), Some(Token { kind: ']', .. }))
             })?;
 
             span = span.merge(inner.span);
@@ -471,16 +506,25 @@ impl<'a> Parser<'a> {
         })
     }
 
-    fn parse_dimension(
+    fn parse_intermediate_value_dimension(
         &mut self,
-        predicate: &dyn Fn(&mut PeekMoreIterator<IntoIter<Token>>) -> bool,
+        predicate: Predicate<'_>,
     ) -> SassResult<Spanned<IntermediateValue>> {
+        let Spanned { node, span } = self.parse_dimension(predicate)?;
+
+        Ok(IntermediateValue::Value(HigherIntermediateValue::Literal(node)).span(span))
+    }
+
+    pub(crate) fn parse_dimension(
+        &mut self,
+        predicate: Predicate<'_>,
+    ) -> SassResult<Spanned<Value>> {
         let Spanned {
             node: val,
             mut span,
         } = self.parse_number(predicate)?;
         let unit = if let Some(tok) = self.toks.peek() {
-            let Token { kind, .. } = *tok;
+            let Token { kind, .. } = tok;
             match kind {
                 'a'..='z' | 'A'..='Z' | '_' | '\\' | '\u{7f}'..=std::char::MAX => {
                     let u = self.parse_identifier_no_interpolation(true)?;
@@ -488,7 +532,7 @@ impl<'a> Parser<'a> {
                     Unit::from(u.node)
                 }
                 '-' => {
-                    if let Some(Token { kind, .. }) = self.toks.peek_next().copied() {
+                    if let Some(Token { kind, .. }) = self.toks.peek_next() {
                         self.toks.reset_cursor();
                         if matches!(kind, 'a'..='z' | 'A'..='Z' | '_' | '\\' | '\u{7f}'..=std::char::MAX)
                         {
@@ -516,28 +560,19 @@ impl<'a> Parser<'a> {
         let n = if val.dec_len == 0 {
             if val.num.len() <= 18 && val.times_ten.is_empty() {
                 let n = Rational64::new_raw(parse_i64(&val.num), 1);
-                return Ok(IntermediateValue::Value(HigherIntermediateValue::Literal(
-                    Value::Dimension(Some(Number::new_small(n)), unit, false),
-                ))
-                .span(span));
+                return Ok(Value::Dimension(Some(Number::new_small(n)), unit, false).span(span));
             }
             BigRational::new_raw(val.num.parse::<BigInt>().unwrap(), BigInt::one())
         } else {
             if val.num.len() <= 18 && val.times_ten.is_empty() {
                 let n = Rational64::new(parse_i64(&val.num), pow(10, val.dec_len));
-                return Ok(IntermediateValue::Value(HigherIntermediateValue::Literal(
-                    Value::Dimension(Some(Number::new_small(n)), unit, false),
-                ))
-                .span(span));
+                return Ok(Value::Dimension(Some(Number::new_small(n)), unit, false).span(span));
             }
             BigRational::new(val.num.parse().unwrap(), pow(BigInt::from(10), val.dec_len))
         };
 
         if val.times_ten.is_empty() {
-            return Ok(IntermediateValue::Value(HigherIntermediateValue::Literal(
-                Value::Dimension(Some(Number::new_big(n)), unit, false),
-            ))
-            .span(span));
+            return Ok(Value::Dimension(Some(Number::new_big(n)), unit, false).span(span));
         }
 
         let times_ten = pow(
@@ -555,14 +590,7 @@ impl<'a> Parser<'a> {
             BigRational::new(BigInt::one(), times_ten)
         };
 
-        Ok(
-            IntermediateValue::Value(HigherIntermediateValue::Literal(Value::Dimension(
-                Some(Number::new_big(n * times_ten)),
-                unit,
-                false,
-            )))
-            .span(span),
-        )
+        Ok(Value::Dimension(Some(Number::new_big(n * times_ten)), unit, false).span(span))
     }
 
     fn parse_paren(&mut self) -> SassResult<Spanned<IntermediateValue>> {
@@ -578,9 +606,9 @@ impl<'a> Parser<'a> {
         }
 
         let mut map = SassMap::new();
-        let key = self.parse_value(true, &|c| {
+        let key = self.parse_value(true, &|parser| {
             matches!(
-                c.peek(),
+                parser.toks.peek(),
                 Some(Token { kind: ':', .. }) | Some(Token { kind: ')', .. })
             )
         })?;
@@ -596,9 +624,9 @@ impl<'a> Parser<'a> {
             Some(..) | None => return Err(("expected \")\".", key.span).into()),
         }
 
-        let val = self.parse_value(true, &|c| {
+        let val = self.parse_value(true, &|parser| {
             matches!(
-                c.peek(),
+                parser.toks.peek(),
                 Some(Token { kind: ',', .. }) | Some(Token { kind: ')', .. })
             )
         })?;
@@ -634,9 +662,9 @@ impl<'a> Parser<'a> {
         }
 
         loop {
-            let key = self.parse_value(true, &|c| {
+            let key = self.parse_value(true, &|parser| {
                 matches!(
-                    c.peek(),
+                    parser.toks.peek(),
                     Some(Token { kind: ':', .. }) | Some(Token { kind: ',', .. })
                 )
             })?;
@@ -644,9 +672,9 @@ impl<'a> Parser<'a> {
             self.expect_char(':')?;
 
             self.whitespace_or_comment();
-            let val = self.parse_value(true, &|c| {
+            let val = self.parse_value(true, &|parser| {
                 matches!(
-                    c.peek(),
+                    parser.toks.peek(),
                     Some(Token { kind: ',', .. }) | Some(Token { kind: ')', .. })
                 )
             })?;
@@ -679,7 +707,7 @@ impl<'a> Parser<'a> {
     fn in_interpolated_identifier_body(&mut self) -> bool {
         match self.toks.peek() {
             Some(Token { kind: '\\', .. }) => true,
-            Some(Token { kind, .. }) if is_name(*kind) => true,
+            Some(Token { kind, .. }) if is_name(kind) => true,
             Some(Token { kind: '#', .. }) => {
                 let next_is_curly = matches!(self.toks.peek_next(), Some(Token { kind: '{', .. }));
                 self.toks.reset_cursor();
@@ -701,9 +729,9 @@ impl<'a> Parser<'a> {
         for _ in 0..6 {
             if let Some(Token { kind, pos }) = self.toks.peek() {
                 if kind.is_ascii_hexdigit() {
-                    span = span.merge(*pos);
-                    self.span_before = *pos;
-                    buf.push(*kind);
+                    span = span.merge(pos);
+                    self.span_before = pos;
+                    buf.push(kind);
                     self.toks.next();
                 } else {
                     break;
@@ -715,8 +743,8 @@ impl<'a> Parser<'a> {
             buf.push('?');
             for _ in 0..(8_usize.saturating_sub(buf.len())) {
                 if let Some(Token { kind: '?', pos }) = self.toks.peek() {
-                    span = span.merge(*pos);
-                    self.span_before = *pos;
+                    span = span.merge(pos);
+                    self.span_before = pos;
                     buf.push('?');
                     self.toks.next();
                 } else {
@@ -743,9 +771,9 @@ impl<'a> Parser<'a> {
                 found_hex_digit = true;
                 if let Some(Token { kind, pos }) = self.toks.peek() {
                     if kind.is_ascii_hexdigit() {
-                        span = span.merge(*pos);
-                        self.span_before = *pos;
-                        buf.push(*kind);
+                        span = span.merge(pos);
+                        self.span_before = pos;
+                        buf.push(kind);
                         self.toks.next();
                     } else {
                         break;
@@ -773,9 +801,9 @@ impl<'a> Parser<'a> {
 
     fn parse_intermediate_value(
         &mut self,
-        predicate: &dyn Fn(&mut PeekMoreIterator<IntoIter<Token>>) -> bool,
+        predicate: Predicate<'_>,
     ) -> Option<SassResult<Spanned<IntermediateValue>>> {
-        if predicate(self.toks) {
+        if predicate(self) {
             return None;
         }
         let (kind, span) = match self.toks.peek() {
@@ -810,7 +838,7 @@ impl<'a> Parser<'a> {
                 }
                 return Some(self.parse_ident_value(predicate));
             }
-            '0'..='9' | '.' => return Some(self.parse_dimension(predicate)),
+            '0'..='9' | '.' => return Some(self.parse_intermediate_value_dimension(predicate)),
             '(' => {
                 self.toks.next();
                 return Some(self.parse_paren());
@@ -825,14 +853,18 @@ impl<'a> Parser<'a> {
                         .span(span)
                 } else {
                     IntermediateValue::Value(HigherIntermediateValue::Literal(
-                        self.super_selectors.last().clone().into_value(),
+                        self.super_selectors
+                            .last()
+                            .clone()
+                            .into_selector()
+                            .into_value(),
                     ))
                     .span(span)
                 }
             }
             '#' => {
                 if let Some(Token { kind: '{', pos }) = self.toks.peek_forward(1) {
-                    self.span_before = *pos;
+                    self.span_before = pos;
                     self.toks.reset_cursor();
                     return Some(self.parse_ident_value(predicate));
                 }
@@ -1045,13 +1077,13 @@ impl<'a> Parser<'a> {
     }
 }
 
-struct IntermediateValueIterator<'a, 'b: 'a> {
-    parser: &'a mut Parser<'b>,
+struct IntermediateValueIterator<'a, 'b: 'a, 'c> {
+    parser: &'a mut Parser<'b, 'c>,
     peek: Option<SassResult<Spanned<IntermediateValue>>>,
-    predicate: &'a dyn Fn(&mut PeekMoreIterator<IntoIter<Token>>) -> bool,
+    predicate: Predicate<'a>,
 }
 
-impl<'a, 'b: 'a> Iterator for IntermediateValueIterator<'a, 'b> {
+impl<'a, 'b: 'a, 'c> Iterator for IntermediateValueIterator<'a, 'b, 'c> {
     type Item = SassResult<Spanned<IntermediateValue>>;
     fn next(&mut self) -> Option<Self::Item> {
         if self.peek.is_some() {
@@ -1062,11 +1094,8 @@ impl<'a, 'b: 'a> Iterator for IntermediateValueIterator<'a, 'b> {
     }
 }
 
-impl<'a, 'b: 'a> IntermediateValueIterator<'a, 'b> {
-    pub fn new(
-        parser: &'a mut Parser<'b>,
-        predicate: &'a dyn Fn(&mut PeekMoreIterator<IntoIter<Token>>) -> bool,
-    ) -> Self {
+impl<'a, 'b: 'a, 'c> IntermediateValueIterator<'a, 'b, 'c> {
+    pub fn new(parser: &'a mut Parser<'b, 'c>, predicate: Predicate<'a>) -> Self {
         Self {
             parser,
             peek: None,
@@ -1127,7 +1156,10 @@ impl<'a, 'b: 'a> IntermediateValueIterator<'a, 'b> {
                                 "/{}",
                                 ValueVisitor::new(self.parser, right.span)
                                     .eval(right.node, false)?
-                                    .to_css_string(right.span)?
+                                    .to_css_string(
+                                        right.span,
+                                        self.parser.options.is_compressed()
+                                    )?
                             ),
                             QuoteKind::None,
                         )),
@@ -1333,7 +1365,7 @@ impl<'a, 'b: 'a> IntermediateValueIterator<'a, 'b> {
                                 "/{}",
                                 ValueVisitor::new(self.parser, val.span)
                                     .eval(val.node, false)?
-                                    .to_css_string(val.span)?
+                                    .to_css_string(val.span, self.parser.options.is_compressed())?
                             ),
                             QuoteKind::None,
                         )),
