@@ -1,13 +1,12 @@
 use std::cmp::Ordering;
 
-use peekmore::PeekMore;
-
 use codemap::{Span, Spanned};
 
 use crate::{
     color::Color,
     common::{Brackets, ListSeparator, Op, QuoteKind},
     error::SassResult,
+    lexer::Lexer,
     parse::Parser,
     selector::Selector,
     unit::Unit,
@@ -104,13 +103,23 @@ impl PartialEq for Value {
                     false
                 }
             }
-            Value::ArgList(list1) => {
-                if let Value::ArgList(list2) = other {
-                    list1 == list2
-                } else {
-                    false
+            Value::ArgList(list1) => match other {
+                Value::ArgList(list2) => list1 == list2,
+                Value::List(list2, ListSeparator::Comma, ..) => {
+                    if list1.len() != list2.len() {
+                        return false;
+                    }
+
+                    for (el1, el2) in list1.iter().zip(list2) {
+                        if &el1.node != el2 {
+                            return false;
+                        }
+                    }
+
+                    true
                 }
-            }
+                _ => false,
+            },
         }
     }
 }
@@ -196,22 +205,28 @@ impl Value {
         }
     }
 
-    pub fn to_css_string(&self, span: Span) -> SassResult<Cow<'static, str>> {
+    pub fn to_css_string(&self, span: Span, is_compressed: bool) -> SassResult<Cow<'static, str>> {
         Ok(match self {
             Value::Important => Cow::const_str("!important"),
             Value::Dimension(num, unit, _) => match unit {
                 Unit::Mul(..) | Unit::Div(..) => {
                     if let Some(num) = num {
-                        return Err(
-                            (format!("{}{} isn't a valid CSS value.", num, unit), span).into()
-                        );
+                        return Err((
+                            format!(
+                                "{}{} isn't a valid CSS value.",
+                                num.to_string(is_compressed),
+                                unit
+                            ),
+                            span,
+                        )
+                            .into());
                     }
 
                     return Err((format!("NaN{} isn't a valid CSS value.", unit), span).into());
                 }
                 _ => {
                     if let Some(num) = num {
-                        Cow::owned(format!("{}{}", num, unit))
+                        Cow::owned(format!("{}{}", num.to_string(is_compressed), unit))
                     } else {
                         Cow::owned(format!("NaN{}", unit))
                     }
@@ -228,17 +243,25 @@ impl Value {
                 Brackets::None => Cow::owned(
                     vals.iter()
                         .filter(|x| !x.is_null())
-                        .map(|x| x.to_css_string(span))
+                        .map(|x| x.to_css_string(span, is_compressed))
                         .collect::<SassResult<Vec<Cow<'static, str>>>>()?
-                        .join(sep.as_str()),
+                        .join(if is_compressed {
+                            sep.as_compressed_str()
+                        } else {
+                            sep.as_str()
+                        }),
                 ),
                 Brackets::Bracketed => Cow::owned(format!(
                     "[{}]",
                     vals.iter()
                         .filter(|x| !x.is_null())
-                        .map(|x| x.to_css_string(span))
+                        .map(|x| x.to_css_string(span, is_compressed))
                         .collect::<SassResult<Vec<Cow<'static, str>>>>()?
-                        .join(sep.as_str()),
+                        .join(if is_compressed {
+                            sep.as_compressed_str()
+                        } else {
+                            sep.as_str()
+                        }),
                 )),
             },
             Value::Color(c) => Cow::owned(c.to_string()),
@@ -278,9 +301,13 @@ impl Value {
             Value::ArgList(args) => Cow::owned(
                 args.iter()
                     .filter(|x| !x.is_null())
-                    .map(|a| a.node.to_css_string(span))
+                    .map(|a| a.node.to_css_string(span, is_compressed))
                     .collect::<SassResult<Vec<Cow<'static, str>>>>()?
-                    .join(", "),
+                    .join(if is_compressed {
+                        ListSeparator::Comma.as_compressed_str()
+                    } else {
+                        ListSeparator::Comma.as_str()
+                    }),
             ),
         })
     }
@@ -317,6 +344,10 @@ impl Value {
         }
     }
 
+    pub fn is_color(&self) -> bool {
+        matches!(self, Value::Color(..))
+    }
+
     pub fn is_special_function(&self) -> bool {
         match self {
             Value::String(s, QuoteKind::None) => is_special_function(s),
@@ -349,11 +380,11 @@ impl Value {
                         num.cmp(&num2.clone().convert(unit2, unit))
                     }
                 }
-                v => {
+                _ => {
                     return Err((
                         format!(
                             "Undefined operation \"{} {} {}\".",
-                            v.inspect(span)?,
+                            self.inspect(span)?,
                             op,
                             other.inspect(span)?
                         ),
@@ -372,7 +403,7 @@ impl Value {
                     ),
                     span,
                 )
-                    .into())
+                    .into());
             }
         })
     }
@@ -457,7 +488,9 @@ impl Value {
                     .collect::<SassResult<Vec<String>>>()?
                     .join(", ")
             )),
-            Value::Dimension(Some(num), unit, _) => Cow::owned(format!("{}{}", num, unit)),
+            Value::Dimension(Some(num), unit, _) => {
+                Cow::owned(format!("{}{}", num.inspect(), unit))
+            }
             Value::Dimension(None, unit, ..) => Cow::owned(format!("NaN{}", unit)),
             Value::ArgList(args) if args.is_empty() => Cow::const_str("()"),
             Value::ArgList(args) if args.len() == 1 => Cow::owned(format!(
@@ -479,7 +512,7 @@ impl Value {
             | Value::True
             | Value::False
             | Value::Color(..)
-            | Value::String(..) => self.to_css_string(span)?,
+            | Value::String(..) => self.to_css_string(span, false)?,
         })
     }
 
@@ -502,7 +535,7 @@ impl Value {
     /// `name` is the argument name. It's used for error reporting.
     pub fn to_selector(
         self,
-        parser: &mut Parser<'_>,
+        parser: &mut Parser,
         name: &str,
         allows_parent: bool,
     ) -> SassResult<Selector> {
@@ -511,12 +544,12 @@ impl Value {
             None => return Err((format!("${}: {} is not a valid selector: it must be a string, a list of strings, or a list of lists of strings.", name, self.inspect(parser.span_before)?), parser.span_before).into()),
         };
         Ok(Parser {
-            toks: &mut string
-                .chars()
-                .map(|c| Token::new(parser.span_before, c))
-                .collect::<Vec<Token>>()
-                .into_iter()
-                .peekmore(),
+            toks: &mut Lexer::new(
+                string
+                    .chars()
+                    .map(|c| Token::new(parser.span_before, c))
+                    .collect::<Vec<Token>>(),
+            ),
             map: parser.map,
             path: parser.path,
             scopes: parser.scopes,
@@ -573,4 +606,8 @@ impl Value {
             _ => return Ok(None),
         }))
     }
+
+    pub fn is_quoted_string(&self) -> bool {
+        matches!(self, Value::String(_, QuoteKind::Quoted))
+    }
 }
